<?php

namespace Drupal\Tests\recently_read\Functional;

use Drupal\Core\Url;
use PHPUnit\Framework\Attributes\Group;

/**
 * Tests record tracking.
 */
#[Group('recently_read')]
final class RecordInsertionTest extends RecentlyReadTestBase {

  /**
   * Test whether records get created correctly.
   *
   * Tests without any module configuration.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function testRecordCreation(): void {
    // Visit all nodes as anonymous user.
    foreach ($this->nodes as $node) {
      $this->drupalGet($node->toUrl());
    }

    $recentlyReadRecords = $this->container->get('entity_type.manager')
      ->getStorage('recently_read')
      ->loadMultiple();

    $this->assertCount(4, $recentlyReadRecords);

    // Test for logged-in users.
    $this->drupalGet(Url::fromRoute('user.login'));
    $this->submitForm([
      'name' => $this->user->getAccountName(),
      'pass' => $this->user->passRaw,
    ],
      'Log in'
      );

    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet($this->nodes[0]->toUrl());
    $this->drupalGet($this->nodes[1]->toUrl());
    $this->drupalGet($this->nodes[2]->toUrl());
    $this->drupalGet($this->nodes[4]->toUrl());

    $recentlyReadRecords = $this->container->get('entity_type.manager')
      ->getStorage('recently_read')
      ->loadByProperties(['user_id' => $this->user->id()]);

    // We visited only 3 nodes that should get tracked as recently read.
    // Confirm that 3 recently read records are found.
    $this->assertCount(3, $recentlyReadRecords);
  }

  /**
   * Test count based record tracking.
   *
   * When this module configuration is set to track certain
   * amount of records per user we need to make sure that
   * the record count always remains as configured.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function testCountBasedRecordTracking(): void {
    $this->config('recently_read.configuration')
      ->set('delete_config', 'count')
      ->set('count', '2')
      ->save();

    // Visit all nodes as anonymous user.
    foreach ($this->nodes as $node) {
      $this->drupalGet($node->toUrl());
    }

    $recentlyReadRecords = $this->container->get('entity_type.manager')
      ->getStorage('recently_read')
      ->loadMultiple();

    // Confirm only 2 records are logged.
    $this->assertCount(2, $recentlyReadRecords);

    // Test with logged-in user.
    $this->drupalGet(Url::fromRoute('user.login'));
    $this->submitForm([
      'name' => $this->user->getAccountName(),
      'pass' => $this->user->passRaw,
    ],
      'Log in'
      );

    $this->drupalGet($this->nodes[0]->toUrl());
    $this->drupalGet($this->nodes[1]->toUrl());
    $this->drupalGet($this->nodes[2]->toUrl());
    $this->drupalGet($this->nodes[4]->toUrl());

    $recentlyReadRecords = $this->container->get('entity_type.manager')
      ->getStorage('recently_read')
      ->loadByProperties(['user_id' => $this->user->id()]);

    // We visited 4 nodes, confirm only 2 records are logged.
    $this->assertCount(2, $recentlyReadRecords);
  }

}
